@@ -24,6 +24,7 @@
 #include <QSettings>
 #include <QTextStream>
 
+
 #define SYNCTEX_GZ_EXT ".synctex.gz"
 #define SYNCTEX_EXT ".synctex"
 
@@ -61,6 +62,7 @@ gswin32c_command=ghostscriptCommand;
 paper_size=psize;
 lastFile=fileName;
 lastPage=1;
+lastScale=0.1;
 fileLoaded=false;
 currentPage=1;
 doc = 0;
@@ -91,7 +93,7 @@ LeftPanelStackedWidget=new QStackedWidget(StructureView);
 
 listpagesWidget=new QListWidget(LeftPanelStackedWidget);
 
-connect(LeftPanelToolBar->addAction(QIcon(":/images/pages.png"),tr("Structure")), SIGNAL(triggered()), this, SLOT(ShowListPages()));
+connect(LeftPanelToolBar->addAction(QIcon(":/images/pages.png"),tr("Pages")), SIGNAL(triggered()), this, SLOT(ShowListPages()));
 LeftPanelStackedWidget->addWidget(listpagesWidget);
 
 LeftPanelToolBar->addSeparator();
@@ -108,6 +110,9 @@ StructureTreeWidget->header()->setStretchLastSection(false);
 connect(LeftPanelToolBar->addAction(QIcon(":/images/structure.png"),tr("Structure")), SIGNAL(triggered()), this, SLOT(ShowStructure()));
 LeftPanelStackedWidget->addWidget(StructureTreeWidget);
 
+showingListPages=true;
+ShowListPages();
+
 LeftPanelLayout->setSpacing(0);
 LeftPanelLayout->setMargin(0);
 LeftPanelLayout->addWidget(LeftPanelToolBar);
@@ -155,9 +160,11 @@ QToolBar *toolBar = addToolBar("Pdf controls");
 toolBar->setObjectName("Pdf controls");
 
 upAct = new QAction(QIcon(":/images/up.png"), tr("Previous"), this);
+upAct->setShortcut(QKeySequence::MoveToPreviousPage);
 toolBar->addAction(upAct);
 
 downAct = new QAction(QIcon(":/images/down.png"), tr("Next"), this);
+downAct->setShortcut(QKeySequence::MoveToNextPage);
 toolBar->addAction(downAct);
 
 toolBar->addSeparator();
@@ -179,8 +186,22 @@ zoomoutAct->setShortcut(QKeySequence::ZoomOut);
 toolBar->addAction(zoomoutAct);
 viewMenu->addAction(zoomoutAct);
 
+viewMenu->addSeparator();
+viewMenu->addAction(StructureView->toggleViewAction());
 
 
+historyBackAct = new QAction(QIcon(":/images/errorprev.png"), tr("Previous Position"), this);
+historyBackAct->setShortcut(QKeySequence::Back);
+//connect(historyBackAct, SIGNAL(triggered()), this, SLOT(historyBack()));
+LeftPanelToolBar->addAction(historyBackAct);
+
+historyForwardAct = new QAction(QIcon(":/images/errornext.png"), tr("Next Position"), this);
+historyForwardAct->setShortcut(QKeySequence::Forward);
+//connect(historyForwardAct, SIGNAL(triggered()), this, SLOT(historyForward()));
+LeftPanelToolBar->addAction(historyForwardAct);
+
+connect( this, SIGNAL( backwardAvailable( bool ) ), historyBackAct, SLOT( setEnabled( bool ) ) );
+connect( this, SIGNAL( forwardAvailable( bool ) ), historyForwardAct, SLOT( setEnabled( bool ) ) );
 
 scaleComboBox = new QComboBox(toolBar);
 zoomCustom = new QLineEdit();
@@ -275,6 +296,7 @@ pdf_file=fn;
 viewpdf_command=ec;
 gswin32c_command=pc;
 int lpage=lastPage;
+qreal lscale=lastScale;
 if (scanner != NULL) 
   {
   synctex_scanner_free(scanner);
@@ -310,18 +332,28 @@ if (doc!=0)
       } 
     QString syncFile = QFileInfo(fn).canonicalFilePath();
     scanner = synctex_scanner_new_with_output_file(syncFile.toUtf8().data(), NULL, 1);
-    initPages();
+
     if ((fn==lastFile) && (lpage <= doc->numPages()) && (lpage>0))
       {
       currentPage=lpage;
+      currentScale=lscale;
+      if ((currentScale < 0.25) || (currentScale > 4)) initPages(true);
+      else initPages(false);
       }
     else 
       {
       currentPage=1;
       lastPage=1;
+      initPages(true);
+      lastScale=currentScale;
       }
+    previousScale=currentScale;
     fileLoaded=true;
     lastFile=fn;
+    setWindowTitle(QFileInfo(fn).fileName());
+    clearHistory();
+    if (showingListPages) ShowListPages();
+    else ShowStructure();
     gotoPage(currentPage);
     } 
     else 
@@ -331,7 +363,7 @@ if (doc!=0)
       }
 }
 
-void PdfViewer::initPages()
+void PdfViewer::initPages(bool checkScale)
 {
 canDisplayPixmap=false;
 disconnectActions();
@@ -345,13 +377,17 @@ listPdfWidgetsStatus.clear();
 PdfDocumentWidget *pdfWidget;
 int pos=0;
 pdfWidget = new PdfDocumentWidget(0,0,doc->page(0));
-pdfWidget->updatePixmap();
-qreal portWidth = scrollArea->viewport()->width();
-QSizeF pageSize = doc->page(0)->pageSizeF();
-if (pageSize.width()!=0) currentScale = (portWidth-30) / pageSize.width()*72.0/pdfWidget->physicalDpiX();
-//(portWidth-50) /  pdfWidget->pixmap()->width();
-if (currentScale < 0.25) currentScale = 0.25;
-else if (currentScale > 4) currentScale = 4;
+if (checkScale)
+    {
+    pdfWidget->updatePixmap();
+    qreal portWidth = scrollArea->viewport()->width();
+    QSizeF pageSize = doc->page(0)->pageSizeF();
+    if (pageSize.width()!=0) currentScale = (portWidth-30) / pageSize.width()*72.0/pdfWidget->physicalDpiX();
+    //(portWidth-50) /  pdfWidget->pixmap()->width();
+    if (currentScale < 0.25) currentScale = 0.25;
+    else if (currentScale > 4) currentScale = 4;
+    lastScale=currentScale;
+    }
 pdfWidget->setScale(currentScale);
 pdfWidget->updatePixmap();
 zoomCustom->setText(QString::number(int(currentScale*100)) + "%");
@@ -367,6 +403,8 @@ if (pdfWidget) delete pdfWidget;
   connect(pdfWidget, SIGNAL(syncpage(int, const QPointF&)), this, SLOT(jumpToEditor(int, const QPointF&)));
   connect(pdfWidget, SIGNAL(updateDone(int)), this, SLOT(updatePageStatus(int)));
   connect(pdfWidget, SIGNAL(gotoDest(int,int,int)), this, SLOT(jumpToDest(int,int,int)));
+  connect(pdfWidget, SIGNAL(pressOnPoint(QPoint)), scrollArea, SLOT(pressHere(QPoint)));
+  connect(pdfWidget, SIGNAL(moveOnPoint(QPoint)), scrollArea, SLOT(moveHere(QPoint)));
   pdfWidget->createblankPixmap(1,heightpix);
   listPdfWidgets.append(pdfWidget);
   listPdfWidgetsPos.append(pos);
@@ -507,6 +545,7 @@ void PdfViewer::connectActions()
 connect(findAct, SIGNAL(triggered()), this, SLOT(enableSearch()));
 connect(upAct, SIGNAL(triggered()), this, SLOT(pageUp()));
 connect(downAct, SIGNAL(triggered()), this, SLOT(pageDown()));
+connect(listpagesWidget, SIGNAL(itemActivated ( QListWidgetItem*)), this, SLOT(slotItemClicked(QListWidgetItem*)));
 connect(listpagesWidget, SIGNAL(itemClicked ( QListWidgetItem*)), this, SLOT(slotItemClicked(QListWidgetItem*)));
 connect(StructureTreeWidget, SIGNAL(itemSelectionChanged()), this, SLOT(ClickedOnStructure()));
 connect(fitWithAct, SIGNAL(triggered()), this, SLOT(fitWidth()));
@@ -519,6 +558,10 @@ connect(searchLineEdit, SIGNAL(returnPressed()), this, SLOT(searchDocument()));
 connect(findButton, SIGNAL(clicked()), this, SLOT(searchDocument()));
 if (scrollArea) connect(scrollArea, SIGNAL(pagezoomOut()), this, SLOT(zoomOut()));
 if (scrollArea) connect(scrollArea, SIGNAL(pagezoomIn()), this, SLOT(zoomIn()));
+connect(historyBackAct, SIGNAL(triggered()), this, SLOT(historyBack()));
+connect(historyForwardAct, SIGNAL(triggered()), this, SLOT(historyForward()));
+//connect( this, SIGNAL( backwardAvailable( bool ) ), historyBackAct, SLOT( setEnabled( bool ) ) );
+//connect( this, SIGNAL( forwardAvailable( bool ) ), historyForwardAct, SLOT( setEnabled( bool ) ) );
 }
 
 void PdfViewer::disconnectActions()
@@ -526,6 +569,7 @@ void PdfViewer::disconnectActions()
 disconnect(findAct, SIGNAL(triggered()), this, SLOT(enableSearch()));
 disconnect(upAct, SIGNAL(triggered()), this, SLOT(pageUp()));
 disconnect(downAct, SIGNAL(triggered()), this, SLOT(pageDown()));
+disconnect(listpagesWidget, SIGNAL(itemActivated ( QListWidgetItem*)), this, SLOT(slotItemClicked(QListWidgetItem*)));
 disconnect(listpagesWidget, SIGNAL(itemClicked ( QListWidgetItem*)), this, SLOT(slotItemClicked(QListWidgetItem*)));
 disconnect(StructureTreeWidget, SIGNAL(itemSelectionChanged()), this, SLOT(ClickedOnStructure()));
 disconnect(fitWithAct, SIGNAL(triggered()), this, SLOT(fitWidth()));
@@ -538,6 +582,10 @@ disconnect(searchLineEdit, SIGNAL(returnPressed()), this, SLOT(searchDocument())
 disconnect(findButton, SIGNAL(clicked()), this, SLOT(searchDocument()));
 if (scrollArea) disconnect(scrollArea, SIGNAL(pagezoomOut()), this, SLOT(zoomOut()));
 if (scrollArea) disconnect(scrollArea, SIGNAL(pagezoomIn()), this, SLOT(zoomIn()));
+disconnect(historyBackAct, SIGNAL(triggered()), this, SLOT(historyBack()));
+disconnect(historyForwardAct, SIGNAL(triggered()), this, SLOT(historyForward()));
+//disconnect( this, SIGNAL( backwardAvailable( bool ) ), historyBackAct, SLOT( setEnabled( bool ) ) );
+//disconnect( this, SIGNAL( forwardAvailable( bool ) ), historyForwardAct, SLOT( setEnabled( bool ) ) );
 }
 
 void PdfViewer::jumpToPdfFromSource(QString sourceFile, int source_line)
@@ -624,6 +672,7 @@ if ((page <= doc->numPages()) && (page>=1))
   disconnect(scrollArea, SIGNAL(doScroll(int)), this, SLOT(checkPage(int)));
   scrollArea->verticalScrollBar()->setValue(listPdfWidgetsPos.at(currentPage-1));
 //  scrollArea->setVisible(0,listPdfWidgetsPos.at(currentPage-1),0,scrollMax);
+  updateHistory(listPdfWidgetsPos.at(currentPage-1));
   updateCurrentPage();
   connect(scrollArea, SIGNAL(doScroll(int)), this, SLOT(checkPage(int)));
   }
@@ -668,12 +717,14 @@ if ((page <= doc->numPages()) && (page>=1))
     int hpos=left-10;
     scrollArea->setVisible(hpos,vpos,20,scrollMax);
     updateCurrentPage();
+    updateHistory(vpos);
   connect(scrollArea, SIGNAL(doScroll(int)), this, SLOT(checkPage(int)));
   }
 }
 
 void PdfViewer::userZoom()
 {
+previousScale=currentScale;
 scaleDocumentZoom(zoomCustom->text());  
 }
 
@@ -683,16 +734,26 @@ if (!fileLoaded) return;
 if (zoom.contains("%")) zoom.remove("%");
 if (zoom.toInt() > 0 && zoom.toInt() <= 400)
   {
+    qreal s=1;
   currentScale=zoom.toFloat() / 100.0;
   if (currentScale < 0.25) currentScale = 0.25;
   else if (currentScale > 4) currentScale = 4;
+  if (previousScale!=0)  s=currentScale/previousScale;
+  int newhpos=(int) (s*(scrollArea->horizontalScrollBar()->value()+scrollArea->viewport()->width()/2)-scrollArea->viewport()->width()/2);
+  int newvpos=(int) (s*(scrollArea->verticalScrollBar()->value()+scrollArea->viewport()->height()/2)-scrollArea->viewport()->height()/2);
+  lastScale=currentScale;
+  previousScale=currentScale;
   //zoomCustom->setText(QString::number(int(currentScale*100)) + "%");
   disconnect(scrollArea, SIGNAL(doRange()), this, SLOT(setScrollMax()));
   disconnect(scrollArea, SIGNAL(doScroll(int)), this, SLOT(checkPage(int)));
   initPagesWithNewScale();
   connect(scrollArea, SIGNAL(doScroll(int)), this, SLOT(checkPage(int)));
   connect(scrollArea, SIGNAL(doRange()), this, SLOT(setScrollMax()));
-  gotoPage(currentPage);
+  //gotoPage(currentPage);
+  updateCurrentPage();
+  scrollArea->horizontalScrollBar()->setValue(newhpos);
+  scrollArea->verticalScrollBar()->setValue(newvpos);
+   clearHistory();
   }
 }
 
@@ -870,19 +931,22 @@ if (currentPage<doc->numPages())
 void PdfViewer::fitWidth()
 {
 if (!fileLoaded) return;
+previousScale=currentScale;
 qreal portWidth = scrollArea->viewport()->width();
 QSizeF	pageSize = doc->page(currentPage-1)->pageSizeF();
 if (pageSize.width()!=0) currentScale = (portWidth-10) / pageSize.width()*72.0/listPdfWidgets.at(currentPage-1)->physicalDpiX();
 else return;
 if (currentScale < 0.25) currentScale = 0.25;
 else if (currentScale > 4) currentScale = 4;
+lastScale=currentScale;
 zoomCustom->setText(QString::number(int(currentScale*100)) + "%");
-userZoom();
+scaleDocumentZoom(zoomCustom->text());
 }
 
 void PdfViewer::fitPage()
 {
 if (!fileLoaded) return;
+previousScale=currentScale;
 qreal portWidth = scrollArea->viewport()->width();
 qreal portHeight = scrollArea->viewport()->height();
 if (scrollArea->horizontalScrollBar()->isVisible()) portHeight+=scrollArea->horizontalScrollBar()->height();
@@ -897,26 +961,31 @@ if (scaleH < scaleW) currentScale=scaleH;
 else currentScale=scaleW ;
 if (currentScale < 0.25) currentScale = 0.25;
 else if (currentScale > 4) currentScale = 4;
+lastScale=currentScale;
 zoomCustom->setText(QString::number(int(currentScale*100)) + "%");
-userZoom();
+scaleDocumentZoom(zoomCustom->text());
 }
 
 void PdfViewer::zoomIn()
 {
 if (!fileLoaded) return;
+previousScale=currentScale;
 currentScale+=0.1;
 if (currentScale > 4) currentScale = 4;
+lastScale=currentScale;
 zoomCustom->setText(QString::number(int(currentScale*100)) + "%");
-userZoom();
+scaleDocumentZoom(zoomCustom->text());
 }
 
 void PdfViewer::zoomOut()
 {
 if (!fileLoaded) return;
+previousScale=currentScale;
 currentScale-=0.1;
 if (currentScale < 0.25) currentScale = 0.25;
+lastScale=currentScale;
 zoomCustom->setText(QString::number(int(currentScale*100)) + "%");
-userZoom();
+scaleDocumentZoom(zoomCustom->text());
 }
 
 void PdfViewer::runExternalViewer()
@@ -1076,12 +1145,14 @@ else QMainWindow::keyPressEvent(e);
 
 void PdfViewer::ShowStructure()
 {
+showingListPages=false;
 LeftPanelStackedWidget->setCurrentWidget(StructureTreeWidget);
 StructureView->setWindowTitle(tr("Structure"));
 }
 
 void PdfViewer::ShowListPages()
 {
+showingListPages=true;
 LeftPanelStackedWidget->setCurrentWidget(listpagesWidget);
 StructureView->setWindowTitle(tr("Pages"));
 }
@@ -1127,3 +1198,51 @@ if (items.count() > 0)
   }
 }
 
+void PdfViewer::historyBack()
+{
+if (stack.count() <= 1) return;
+// Update the history entry
+forwardStack.push(scrollArea->verticalScrollBar()->value());
+stack.pop(); // throw away the old version of the current entry
+scrollArea->verticalScrollBar()->setValue(stack.top()); // previous entry
+updateCurrentPage();
+emit backwardAvailable(stack.count() > 1);
+emit forwardAvailable(true);
+}
+
+void PdfViewer::historyForward()
+{
+if (forwardStack.isEmpty()) return;
+if (!stack.isEmpty()) 
+  {
+    // Update the history entry
+    stack.top() =scrollArea->verticalScrollBar()->value();
+  }
+stack.push(forwardStack.pop());
+scrollArea->verticalScrollBar()->setValue(stack.top()); // previous entry
+updateCurrentPage();
+emit backwardAvailable(true);
+emit forwardAvailable(!forwardStack.isEmpty());
+}
+
+void PdfViewer::clearHistory()
+{
+forwardStack.clear();
+if (!stack.isEmpty()) 
+  {
+  int tmp=stack.top();
+  stack.resize(0);
+  stack.push(tmp);
+  }
+emit forwardAvailable(false);
+emit backwardAvailable(false);
+}
+
+void PdfViewer::updateHistory(int pos)
+{
+if (!stack.isEmpty() && stack.top()==pos) return;
+stack.push(pos);
+emit backwardAvailable(stack.count() > 1);
+emit forwardAvailable(!forwardStack.isEmpty());
+}
+
